/*
procesorowiec@gmail.com
*/

//---.cpp file---
#include "AVT5636lib.h"
#include "Arduino.h"

const int outNum2Pin[] = {0, OUTPUT1_PIN, OUTPUT2_PIN, OUTPUT3_PIN, OUTPUT4_PIN};
const int inNum2Pin[] = {0, INPUT1_PIN, INPUT2_PIN, INPUT3_PIN, INPUT4_PIN};
const int adcNum2Pin[] = {0, A0, A1, A2, A3};

String version;

//---constructor------------------
AVT5636::AVT5636(){
	version = AVT5636LIB_VER;
}

//---init-------------------------
void AVT5636::init(){
  //init outputs
  digitalWrite(OUTPUT1_PIN, OUTPUT_IDLE_LEVEL);
  digitalWrite(OUTPUT2_PIN, OUTPUT_IDLE_LEVEL);
  digitalWrite(OUTPUT3_PIN, OUTPUT_IDLE_LEVEL);
  digitalWrite(OUTPUT4_PIN, OUTPUT_IDLE_LEVEL);
  pinMode(OUTPUT1_PIN, OUTPUT);
  pinMode(OUTPUT2_PIN, OUTPUT);
  pinMode(OUTPUT3_PIN, OUTPUT);
  pinMode(OUTPUT4_PIN, OUTPUT);
    
  //init leds
  //leds the occupy the same pins as outputs
  
  //init pulse
  digitalWrite(PULSE1_PIN, PULSE_IDLE_LEVEL);
  digitalWrite(PULSE2_PIN, PULSE_IDLE_LEVEL);
  pinMode(PULSE1_PIN, OUTPUT);
  pinMode(PULSE2_PIN, OUTPUT);
  
  //init buzzer
  digitalWrite(BUZZER_PIN, BUZZER_IDLE_LEVEL);
  pinMode(BUZZER_PIN, OUTPUT);
  
  //init buttons
  pinMode(S1_PIN, INPUT_PULLUP);
  pinMode(S2_PIN, INPUT_PULLUP);
  pinMode(S3_PIN, INPUT_PULLUP);
  pinMode(S4_PIN, INPUT_PULLUP);

  //init inputs
  pinMode(INPUT1_PIN, INPUT);
  pinMode(INPUT2_PIN, INPUT);
  pinMode(INPUT3_PIN, INPUT);
  pinMode(INPUT4_PIN, INPUT);
  
  //lcd
  Serial.begin(19200);
  delay(200);
  lcdClear();
}
  
//---Outputs--------------------------
void AVT5636::outOn(int num){
  if ((num > 0) && (num <= 4)){
	  
	int pin = outNum2Pin[num];
	digitalWrite(pin, OUTPUT_ACTIVE_LEVEL);    
  }
}

void AVT5636::outOff(int num){
  if ((num > 0) && (num <= 4)){
	int pin = outNum2Pin[num];
	digitalWrite(pin, OUTPUT_IDLE_LEVEL);    
  }
}

void AVT5636::outPulse(int num, int tIme){
  outOn(num);
  delay(tIme);
  outOff(num);
}

bool AVT5636::outGet(int num){
  if ((num > 0) && (num <= 4)){
	int pin = outNum2Pin[num];
	if (digitalRead(pin) == OUTPUT_ACTIVE_LEVEL) return OUTPUT_ACTIVE_LEVEL;
  }
  return OUTPUT_IDLE_LEVEL;
}

int AVT5636::outputsGet(){
  int port = PORTB;
  port >>= 2;
  return (port & 0x0F);
}

void AVT5636::outputsSet(int state){
	int port = PORTB;
	port &= ~(0x3C);
	state <<= 2;
	port |= state;
	PORTB = (char)port;
}

//---leds---------------------------
void AVT5636::ledOn(int num){
  outOn(num);  
}

void AVT5636::ledOff(int num){
  outOff(num);
}

void AVT5636::ledPulse(int num, int tIme){
  outOn(num); 
  delay(tIme);
  outOff(num);
}

//---pulses---------------------------
void AVT5636::pulse1Set(int val){
  if (val <= 255){
	analogWrite(PULSE1_PIN, val);
  }
}

void AVT5636::pulse2Set(int val){
  if (val <= 255){
	analogWrite(PULSE2_PIN, val);
  }
}

//---buzzer-------------------------
void AVT5636::buzzerOn(){
  digitalWrite(BUZZER_PIN, BUZZER_ACTIVE_LEVEL);  
}

void AVT5636::buzzerOff(){
  digitalWrite(BUZZER_PIN, BUZZER_IDLE_LEVEL);  
}

void AVT5636::buzzerPulse(int tIme){
  buzzerOn();
  delay(tIme);
  buzzerOff();
}

//---buttons------------------------
int AVT5636::getButtons(){
  int result = 0;
  if (digitalRead(S1_PIN) == SWITCH_ACTIVE_LEVEL) result = AVT5636_S1;  
  if (digitalRead(S2_PIN) == SWITCH_ACTIVE_LEVEL) result = AVT5636_S2;  
  if (digitalRead(S3_PIN) == SWITCH_ACTIVE_LEVEL) result = AVT5636_S3;  
  if (digitalRead(S4_PIN) == SWITCH_ACTIVE_LEVEL) result = AVT5636_S4;
  return result;  
}

//---inputs------------------------
int AVT5636::getInputs(){
  int result = PINC & (0x0F);
  return result;  
}

bool AVT5636::getInput(int num){
  if ((num > 0) && (num <= 4)){
	int pin = inNum2Pin[num];
	if (digitalRead(pin) == HIGH) return 1;
  }	
  return 0;  
}

//---analog-------------------------
int AVT5636::getInputAdc(int num){
  if ((num > 0) && (num <= 4)){
	int pin = adcNum2Pin[num];
	return analogRead(pin);
  }	
  return 0;  
}

int AVT5636::getInputVoltage(int num){
  float v = (5000.0 / 1023.0);	
  v *= getInputAdc(num);
  return (int)v;				//mV
}

int AVT5636::getInputPercent(int num){
  float v = (100.0 / 1023.0);
  v *= getInputAdc(num);
  return (int)v;				//%
}

//---components-------------------------
int AVT5636::getTouch(){
  return getInput(AVT5636_TOUCH);
}

int AVT5636::getVreg(){
  return getInputVoltage(AVT5636_VREG);
}

int AVT5636::getLight(){
  return (100 - getInputPercent(AVT5636_FOTO));
} 

int AVT5636::getTemp(){	
  return (getInputVoltage(AVT5636_TEMP) / 10);
}

int AVT5636::getVin(){	
  return getInputVoltage(AVT5636_VIN);
}

int AVT5636::getVin10(){	
  return (getInputVoltage(AVT5636_VIN) * 10);
}


//---LCD-------------------------
void AVT5636::lcdPrint(String str){
  String buff = "@LCD=";
  buff = buff + str + "\r";
  Serial.print(buff);	 
  delay(20);
}

void AVT5636::lcdPosition(int y, int x){
  String buff = "@LCD_";
  buff = buff + y + "," + x + "\r";
  Serial.print(buff);
  delay(20);
}

void AVT5636::lcdClear(){
  String buff = "@LCD!";
  buff = buff + "\r";
  Serial.print(buff);
  delay(20);
}

void AVT5636::lcdSpecChar(int index, char *src){
  String buff = "@LCD*";
  buff = buff + index + "," + src + "\r";
  Serial.print(buff);
  delay(20);
}

void AVT5636::lcdMessage(int y, int x, String str){
  lcdPosition(y,x);
  lcdPrint(str);
}





